// ---------------------------------------------------------------------------
//	M88 - PC-88 Emulator
//	Copyright (C) cisc 1998, 1999.
// ---------------------------------------------------------------------------
//	generic file io class for Win32
// ---------------------------------------------------------------------------
//	$Id: file.cpp,v 1.6 1999/12/28 11:14:05 cisc Exp $

#include "headers.h"
#include "strcv.h"		//	add by TAN-Y
#include "File.h"

char FileIO::m_szCurrentDirectory[MAX_PATH];

// ---------------------------------------------------------------------------
//	\z/
// ---------------------------------------------------------------------------

FileIO::FileIO()
{
	flags = 0;
}

FileIO::FileIO(const char* filename, uint flg)
{
	flags = 0;
	Open(filename, flg);
}

FileIO::~FileIO()
{
	Close();
}

// ---------------------------------------------------------------------------
//	t@CJ
// ---------------------------------------------------------------------------

bool FileIO::Open(const char* filename, uint flg)
{
	Close();

//	strncpy(path, filename, MAX_PATH);

//	strcpy( path, "\\My Documents\\devtest\\" );
//	strcat( path, filename );

	GetFullPath( path, filename );

	DWORD access = (flg & readonly ? 0 : GENERIC_WRITE) | GENERIC_READ;
	DWORD share = (flg & readonly) ? FILE_SHARE_READ : 0;
	DWORD creation = flg & create ? CREATE_ALWAYS : OPEN_EXISTING;

//	LPWSTR szWFileName = MultiToWide( filename );
	LPWSTR szWFileName = MultiToWide( path );
	hfile = CreateFile(szWFileName, access, share, 0, creation, 0, 0);
	free( szWFileName );

	flags = (flg & readonly) | (hfile == INVALID_HANDLE_VALUE ? 0 : open);
	if (!(flags & open))
	{
		switch (GetLastError())
		{
		case ERROR_FILE_NOT_FOUND:		error = file_not_found; break;
		case ERROR_SHARING_VIOLATION:	error = sharing_violation; break;
		default: error = unknown; break;
		}
	}
	SetLogicalOrigin(0);

	return !!(flags & open);
}

// ---------------------------------------------------------------------------
//	t@CȂꍇ͍쐬
// ---------------------------------------------------------------------------

bool FileIO::CreateNew(const char* filename)
{
	Close();

//	strncpy(path, filename, MAX_PATH);

//	strcpy( path, "\\My Documents\\devtest\\" );
//	strcat( path, filename );

	GetFullPath( path, filename );

	DWORD access = GENERIC_WRITE | GENERIC_READ;
	DWORD share = 0;
	DWORD creation = CREATE_NEW;

//	LPWSTR szWFileName = MultiToWide( filename );
	LPWSTR szWFileName = MultiToWide( path );
	hfile = CreateFile(szWFileName, access, share, 0, creation, 0, 0);
	free( szWFileName );

	flags = (hfile == INVALID_HANDLE_VALUE ? 0 : open);
	SetLogicalOrigin(0);

	return !!(flags & open);
}

// ---------------------------------------------------------------------------
//	t@C蒼
// ---------------------------------------------------------------------------

bool FileIO::Reopen(uint flg)
{
	if (!(flags & open)) return false;
	if ((flags & readonly) && (flg & create)) return false;

	if (flags & readonly) flg |= readonly;

	Close();

	DWORD access = (flg & readonly ? 0 : GENERIC_WRITE) | GENERIC_READ;
	DWORD share = flg & readonly ? FILE_SHARE_READ : 0;
	DWORD creation = flg & create ? CREATE_ALWAYS : OPEN_EXISTING;

	LPWSTR szWPath = MultiToWide( path );
	hfile = CreateFile(szWPath, access, share, 0, creation, 0, 0);
	free( szWPath );
	
	flags = (flg & readonly) | (hfile == INVALID_HANDLE_VALUE ? 0 : open);
	SetLogicalOrigin(0);

	return !!(flags & open);
}

// ---------------------------------------------------------------------------
//	t@C
// ---------------------------------------------------------------------------

void FileIO::Close()
{
	if (GetFlags() & open)
	{
		CloseHandle(hfile);
		flags = 0;
	}
}

// ---------------------------------------------------------------------------
//	t@Ck̓ǂݏo
// ---------------------------------------------------------------------------

int32 FileIO::Read(void* dest, int32 size)
{
	if (!(GetFlags() & open))
		return -1;
	
	DWORD readsize;
	if (!ReadFile(hfile, dest, size, &readsize, 0))
		return -1;
	return readsize;
}

// ---------------------------------------------------------------------------
//	t@Cւ̏o
// ---------------------------------------------------------------------------

int32 FileIO::Write(const void* dest, int32 size)
{
	if (!(GetFlags() & open) || (GetFlags() & readonly))
		return -1;
	
	DWORD writtensize;
	if (!WriteFile(hfile, dest, size, &writtensize, 0))
		return -1;
	return writtensize;
}

// ---------------------------------------------------------------------------
//	t@CV[N
// ---------------------------------------------------------------------------

bool FileIO::Seek(int32 pos, SeekMethod method)
{
	if (!(GetFlags() & open))
		return false;
	
	DWORD wmethod;
	switch (method)
	{
	case begin:	
		wmethod = FILE_BEGIN; pos += lorigin; 
		break;
	case current:	
		wmethod = FILE_CURRENT; 
		break;
	case end:		
		wmethod = FILE_END; 
		break;
	default:
		return false;
	}

	return 0xffffffff != SetFilePointer(hfile, pos, 0, wmethod);
}

// ---------------------------------------------------------------------------
//	t@C̈ʒu𓾂
// ---------------------------------------------------------------------------

int32 FileIO::Tellp()
{
	if (!(GetFlags() & open))
		return 0;

	return SetFilePointer(hfile, 0, 0, FILE_CURRENT) - lorigin;
}

// ---------------------------------------------------------------------------
//	݂̈ʒut@C̏I[Ƃ
// ---------------------------------------------------------------------------

bool FileIO::SetEndOfFile()
{
	if (!(GetFlags() & open))
		return false;
	return ::SetEndOfFile(hfile) != 0;
}


/*---------------------------------------------------------------------------
		JgfBNgݒ肷
---------------------------------------------------------------------------*/

void FileIO::SetCurrentDirectory( const char *szDirectory )
{
	strcpy( m_szCurrentDirectory, szDirectory );
}


/*---------------------------------------------------------------------------
		tpX擾i蔲Łj
		o͐̃obt@TCY MAX_PATH ƂB
---------------------------------------------------------------------------*/

void FileIO::GetFullPath( char *szDst, const char *szFileName )
{
	if( *szFileName == '\\' )
	{
		//
		//	̃t@C̐擪 \ 猳XtpXƔf(
		//
		strcpy( szDst, szFileName );
		return;
	}

	//
	//	łȂ΃JgfBNgƌB
	//	΃pXƂ͍lĂȂł....(^^;
	//
	int iCurDirLen = strlen( m_szCurrentDirectory ) + 1;
	int iFileNameLen = strlen( szFileName );
	int iTotalLen = iCurDirLen + iFileNameLen;

	if( iTotalLen > MAX_PATH )
	{
		iFileNameLen = MAX_PATH - iCurDirLen;
	}

	strcpy( szDst, m_szCurrentDirectory );
	strcat( szDst, "\\" );
	strncpy( szDst + iCurDirLen, szFileName, iFileNameLen );
	*( szDst + iCurDirLen + iFileNameLen ) = 0;
}


